@@ -36,7 +36,6 @@
 #include <strings.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#include "wireless-helper.h"
 #include <unistd.h>
 #include <sys/socket.h>
 
@@ -434,6 +433,7 @@ applet_do_notify (NMApplet *applet,
 {
 	NotifyNotification *notify;
 	GError *error = NULL;
+	char *escaped;
 
 	g_return_if_fail (applet != NULL);
 	g_return_if_fail (summary != NULL);
@@ -446,8 +446,12 @@ applet_do_notify (NMApplet *applet,
 
 	applet_clear_notify (applet);
 
-	notify = notify_notification_new (summary, message,
-	                                  icon ? icon : GTK_STOCK_NETWORK, NULL);
+	escaped = utils_escape_notify_message (message);
+	notify = notify_notification_new (summary,
+	                                  escaped,
+	                                  icon ? icon : GTK_STOCK_NETWORK,
+	                                  NULL);
+	g_free (escaped);
 	applet->notification = notify;
 
 	notify_notification_attach_to_status_icon (notify, applet->status_icon);
@@ -460,8 +464,9 @@ applet_do_notify (NMApplet *applet,
 	}
 
 	if (!notify_notification_show (notify, &error)) {
-		g_warning ("Failed to show notification: %s", error->message);
-		g_error_free (error);
+		g_warning ("Failed to show notification: %s",
+		           error && error->message ? error->message : "(unknown)");
+		g_clear_error (&error);
 	}
 }
 
@@ -662,58 +667,6 @@ make_vpn_disconnection_message (NMVPNConnection *vpn,
 	return g_strdup_printf (_("\nThe VPN connection '%s' disconnected."), nm_setting_connection_get_id (s_con));
 }
 
-typedef struct {
-	const char *tag;
-	const char *replacement;
-} Tag;
-
-static Tag banner_tags[] = {
-	{ "<center>", NULL },
-	{ "</center>", NULL },
-	{ "<p>", "\n" },
-	{ "</p>", NULL },
-	{ "<B>", "<b>" },
-	{ "</B>", "</b>" },
-	{ "<I>", "<i>" },
-	{ "</I>", "</i>" },
-	{ "<u>", "<u>" },
-	{ "</u>", "</u>" },
-	{ NULL, NULL }
-};
-
-static char *
-construct_vpn_banner (const char *src)
-{
-	const char *p = src;
-	GString *banner;
-
-	/* Filter the banner text and get rid of some HTML tags since the
-	 * notification spec only allows a subset of HTML.
-	 */
-
-	banner = g_string_sized_new (strlen (src) + 5);
-	g_string_append_c (banner, '\n');
-	while (*p) {
-		Tag *t = &banner_tags[0];
-		gboolean found = FALSE;
-
-		while (t->tag) {
-			if (strncasecmp (p, t->tag, strlen (t->tag)) == 0) {
-				p += strlen (t->tag);
-				if (t->replacement)
-					g_string_append (banner, t->replacement);
-				found = TRUE;
-				break;
-			}
-			t++;
-		}
-		if (!found)
-			g_string_append_c (banner, *p++);
-	}
-
-	return g_string_free (banner, FALSE);
-}
-
 static void
 vpn_connection_state_changed (NMVPNConnection *vpn,
                               NMVPNConnectionState state,
@@ -723,7 +676,7 @@ vpn_connection_state_changed (NMVPNConnection *vpn,
 	NMApplet *applet = NM_APPLET (user_data);
 	NMConnection *connection;
 	const char *banner;
-	char *title = NULL, *msg = NULL;
+	char *title = NULL, *msg;
 	gboolean device_activating, vpn_activating;
 
 	device_activating = applet_is_any_device_activating (applet);
@@ -743,7 +696,7 @@ vpn_connection_state_changed (NMVPNConnection *vpn,
 		banner = nm_vpn_connection_get_banner (vpn);
 		if (banner && strlen (banner)) {
 			title = _("VPN Login Message");
-			msg = construct_vpn_banner (banner);
+			msg = g_strdup_printf ("%s\n", banner);
 			applet_do_notify (applet, NOTIFY_URGENCY_LOW, title, msg,
 			                  "gnome-lockscreen", NULL, NULL, NULL, NULL);
 			g_free (msg);
@@ -1352,6 +1305,30 @@ nma_set_networking_enabled_cb (GtkWidget *widget, NMApplet *applet)
 	nm_client_sleep (applet->nm_client, !state);
 }
 
+
+static void
+nma_set_notifications_enabled_cb (GtkWidget *widget, NMApplet *applet)
+{
+	gboolean state;
+
+	g_return_if_fail (applet != NULL);
+
+	state = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget));
+
+	gconf_client_set_bool (applet->gconf_client,
+	                       PREF_DISABLE_CONNECTED_NOTIFICATIONS,
+	                       !state,
+	                       NULL);
+	gconf_client_set_bool (applet->gconf_client,
+	                       PREF_DISABLE_DISCONNECTED_NOTIFICATIONS,
+	                       !state,
+	                       NULL);
+	gconf_client_set_bool (applet->gconf_client,
+	                       PREF_SUPPRESS_WIRELESS_NETWORKS_AVAILABLE,
+	                       !state,
+	                       NULL);
+}
+
 /*
  * nma_menu_show_cb
  *
@@ -1463,6 +1440,7 @@ nma_context_menu_update (NMApplet *applet)
 	NMState state;
 	gboolean have_wireless = FALSE;
 	gboolean wireless_hw_enabled;
+	gboolean notifications_enabled = TRUE;
 
 	state = nm_client_get_state (applet->nm_client);
 
@@ -1492,6 +1470,17 @@ nma_context_menu_update (NMApplet *applet)
 	gtk_widget_set_sensitive (GTK_WIDGET (applet->wifi_enabled_item),
 	                          wireless_hw_enabled);
 
+	/* Enabled notifications */
+	g_signal_handler_block (G_OBJECT (applet->notifications_enabled_item),
+	                        applet->notifications_enabled_toggled_id);
+	if (   gconf_client_get_bool (applet->gconf_client, PREF_DISABLE_CONNECTED_NOTIFICATIONS, NULL)
+	    && gconf_client_get_bool (applet->gconf_client, PREF_DISABLE_DISCONNECTED_NOTIFICATIONS, NULL)
+	    && gconf_client_get_bool (applet->gconf_client, PREF_SUPPRESS_WIRELESS_NETWORKS_AVAILABLE, NULL))
+		notifications_enabled = FALSE;
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (applet->notifications_enabled_item), notifications_enabled);
+	g_signal_handler_unblock (G_OBJECT (applet->notifications_enabled_item),
+	                          applet->notifications_enabled_toggled_id);
+
 	/* Don't show wifi-specific stuff if wireless is off */
 	if (state != NM_STATE_ASLEEP) {
 		const GPtrArray *devices;
@@ -1580,6 +1569,17 @@ static GtkWidget *nma_context_menu_create (NMApplet *applet)
 
 	nma_menu_add_separator_item (GTK_WIDGET (menu));
 
+	/* Toggle notifications item */
+	applet->notifications_enabled_item = gtk_check_menu_item_new_with_mnemonic (_("Enable N_otifications"));
+	id = g_signal_connect (applet->notifications_enabled_item,
+	                       "toggled",
+	                       G_CALLBACK (nma_set_notifications_enabled_cb),
+	                       applet);
+	applet->notifications_enabled_toggled_id = id;
+	gtk_menu_shell_append (menu, applet->notifications_enabled_item);
+
+	nma_menu_add_separator_item (GTK_WIDGET (menu));
+
 	/* 'Connection Information' item */
 	applet->info_menu_item = gtk_image_menu_item_new_with_mnemonic (_("Connection _Information"));
 	g_signal_connect_swapped (applet->info_menu_item,
@@ -1688,9 +1688,14 @@ applet_get_exported_connection_for_device (NMDevice *device, NMApplet *applet)
 		const GPtrArray *devices;
 
 		active = g_ptr_array_index (active_connections, i);
+		if (!active)
+			continue;
+
 		devices = nm_active_connection_get_devices (active);
 		service_name = nm_active_connection_get_service_name (active);
 		connection_path = nm_active_connection_get_connection (active);
+		if (!devices || !service_name || !connection_path)
+			continue;
 
 		if (strcmp (service_name, NM_DBUS_SERVICE_USER_SETTINGS) != 0)
 			continue;
@@ -2665,15 +2670,11 @@ constructor (GType type,
 
 	applet->dbus_settings = (NMDBusSettings *) nm_dbus_settings_system_new (applet_dbus_manager_get_connection (dbus_mgr));
 
-	applet->gconf_settings = nma_gconf_settings_new ();
+	applet->gconf_settings = nma_gconf_settings_new (applet_dbus_manager_get_connection (dbus_mgr));
 	g_signal_connect (applet->gconf_settings, "new-secrets-requested",
 	                  G_CALLBACK (applet_settings_new_secrets_requested_cb),
 	                  applet);
 
-	dbus_g_connection_register_g_object (applet_dbus_manager_get_connection (dbus_mgr),
-	                                     NM_DBUS_PATH_SETTINGS,
-	                                     G_OBJECT (applet->gconf_settings));
-
 	/* Start our DBus service */
 	if (!applet_dbus_manager_start_service (dbus_mgr)) {
 		g_object_unref (applet);
